import '../styles/globals.scss';
import ContentLayout from '../shared/layout-components/layout/content-layout';
import Authenticationlayout from "../shared/layout-components/layout/authentication-layout";
import Landinglayout from '@/shared/layout-components/layout/landing-layout';
import { UserProvider } from '@/pages/components/providers/user-context';
import { parseCookies } from "nookies";
import Router  from 'next/router';


const layouts:any = {

  Contentlayout: ContentLayout,
  Landinglayout: Landinglayout,
  Authenticationlayout: Authenticationlayout,

};

const redirectUser = (ctx:any, location:any) => {
  if (ctx.req) {
    ctx.res.writeHead(302, { Location: location });
    ctx.res.end();
  } else {
    Router.replace(location)
  }
};


function MyApp(props:any) {
  const { Component, pageProps } = props;
  const Layout = layouts[Component.layout] || ((pageProps: any) => <Component>{pageProps}</Component>);

  return (

    <Layout>
      <UserProvider initUser={pageProps.user}>
      <Component {...pageProps} />
      </UserProvider>
    </Layout>
    
  )
}

export default MyApp;
MyApp.getInitialProps = async (aa:any) => {
  const {Component, ctx} = aa
  const { token, user } = parseCookies(ctx);
  let pageProps = {};

  if (Component.getInitialProps) {
    pageProps = await Component.getInitialProps(ctx);
  }

  if (!(token && user)) {
    // if a user not logged in then user can't access those pages
    const isProtectedRoute =
      ctx.pathname === "/profile/dashboard" ||
      ctx.pathname === "/profile/chat/[id]" ||
      ctx.pathname === "/requirements" ||
      ctx.pathname === "/requirements/post" ||
      ctx.pathname === '/post-service'
    if (isProtectedRoute) {
      redirectUser(ctx, "/login");
    }
  } else {
    // if a user logged in then user can't access those pages
    const ifLoggedIn =
      ctx.pathname === "/login" ||
      ctx.pathname === "/register";
    if (ifLoggedIn) {
      redirectUser(ctx, "/");
    }
    pageProps.user = JSON.parse(user);
  }
  return {
    pageProps,
  };
};